package de.espirit.firstspirit.opt.example.webedit.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.webedit.plugin.report.ParameterMap;
import de.espirit.firstspirit.webedit.plugin.report.ReportPluginProvider;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.ResourceBundle;


/**
 * Defines a data provider operating on {@link TextBlocksObject}, to be used in conjunction with {@link
 * TextBlocksWebeditReportPlugin}.
 * <p/>
 * Fields are used heavily to keep track of the entire set of results (based on filter configuration) that could be
 * returned by {@code getNext()}, as well as the {@code _offset} in {@code _textBlocks}, marking the location within
 * that list up to which {@code getNext()} has already returned results during the lifetime of this object.
 */
public class TextBlocksReportPluginProvider implements ReportPluginProvider<TextBlocksObject> {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.report.ReportMessages";

	private BaseContext _context;

	private boolean _running;
	private int _offset;
	private int _total;

	private ArrayList<TextBlocksObject> _textBlocks;

	private String _patternSearchTerm = null;
	private Boolean _patternSearchTitlesOnly = false;
	private String _patternSorting = "ascending";

	private ResourceBundle _resourceBundle;


	/**
	 * Constructor. Stores the context {@link TextBlocksWebeditReportPlugin} operates in and configures a resource bundle
	 * for localization, using the WebClient's user interface locale.
	 *
	 * @param context The context the report plug-in operates in.
	 */
	TextBlocksReportPluginProvider(@NotNull final BaseContext context) {
		_context = context;
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());
	}


	/**
	 * Basic initialization. Stores the report plug-in's filter parameter input in fields, requests a {@link List} of text
	 * blocks that match the current filter configuration (see {@code getTextBlocksList()} and sets the total number of
	 * results that may be returned by {@code getNext()} as well as the current offset (from the start of {@code
	 * _textBlocks}) to 0.
	 *
	 * @param parameter Report parameter values (see {@link de.espirit.firstspirit.webedit.plugin.WebeditReportPlugin}).
	 */
	@Override
	public void start(@NotNull final ParameterMap parameter) {
		_patternSearchTerm = parameter.get(TextBlocksWebeditReportPlugin.PATTERN_TEXT);
		_patternSearchTitlesOnly = parameter.get(TextBlocksWebeditReportPlugin.PATTERN_BOOLEAN);
		_patternSorting = parameter.get(TextBlocksWebeditReportPlugin.PATTERN_SELECT);
		_textBlocks = getSortedTextBlocksList();
		_total = _textBlocks.size();
		_offset = 0;
		_running = true;
	}


	/**
	 * Provides a list of results to return to the WebClient, up to {@code count} items.
	 * <p/>
	 * This method operates on the field {@code _textBlocks} that was filled in {@code start()}, and collects {@code
	 * _textBlocks} indices [{@code _offset} .. ({@code _offset} + {@code count}] in {@code results}. Lastly, {@code
	 * _offset} is increased by the size of {@code results}, and the operational status of this data provider instance is
	 * checked: if {@code _offset} has surpassed the total number of available results, or if less results can be returned
	 * than were requested by WebClient, all available results have been returned to the WebClient, and further calls to
	 * {@code getNext()} would be in vain, so {@code _running} is set to {@code false}.
	 *
	 * @param count The requested count of elements.
	 * @return A {@link List} of {@link TextBlocksObject}, with less than or equal to {@code count} entries.
	 */
	@NotNull
	@Override
	public List<TextBlocksObject> getNext(final int count) {

		if ((count <= 0)) {
			_total = 0;
			_running = false;
			return Collections.emptyList();
		}

		final List<TextBlocksObject> results = new ArrayList<TextBlocksObject>();
		int i = _offset;
		if (i > _textBlocks.size()) {
			_total = 0;
			_running = false;
			return Collections.emptyList();
		} else {
			while ((i < _offset + count) && (i < _textBlocks.size())) {
				results.add(_textBlocks.get(i));
				i++;
			}
			_total = _textBlocks.size();
		}

		_offset += results.size();
		if (_offset >= _total || results.size() < count) {
			_running = false;
		}

		return results;
	}


	/**
	 * Provides the operational status of this data provider. If {@code getNext()} has not returned all possible results
	 * yet, the running status should be {@code true}, otherwise {@code false}.
	 *
	 * @return {@code true} if the provider should be considered "running" ({@code getNext()} can still provide more
	 *         results) or "stopped" (all results than can be returned have been provided to WebClient).
	 */
	@Override
	public boolean isRunning() {
		return _running;
	}


	/**
	 * Provides the total number of text blocks that match the current filter configuration.
	 *
	 * @return The total number of results that could be displayed, based on the current filter configuration.
	 */
	@Override
	public int getTotal() {
		return _total;
	}


	/**
	 * Set the operational status of this provider to "stopped".
	 */
	@Override
	public void close() {
		_total = 0;
		_running = false;
	}


	/**
	 * Assembles a {@link List} of {@link TextBlocksObject}s, using sorting and selection criteria defined by the parameter
	 * fields {@code _patternSorting}, {@code _patternSearchTerm} and {@code _patternSearchTitlesOnly}.
	 *
	 * @return A filtered and sorted {@link List} of {@link TextBlocksObject}s.
	 */
	@NotNull
	private ArrayList<TextBlocksObject> getSortedTextBlocksList() {
		final ArrayList<TextBlocksObject> textBlocks = new ArrayList<TextBlocksObject>();
		for (int j = 1; j <= 11; j++) {
			if (_patternSorting != null && "ascending".equals(_patternSorting)) {
				for (int i = 1; i <= 11; i++) {
					getTextBlocks(textBlocks, i);
				}
			} else {
				for (int i = 11; i > 0; i--) {
					getTextBlocks(textBlocks, i);
				}
			}
		}
		return textBlocks;
	}


	/**
	 * Assembles a {@link List} of {@link TextBlocksObject}s, using selection criteria defined by the parameter fields
	 * {@code _patternSearchTerm} and {@code _patternSearchTitlesOnly}.
	 * <p/>
	 * In order to simulate delay due to data requests from a remote web service (and thereby make dynamic loading of more
	 * results during continuous scrolling easily visible in the WebClient interface), short sleep periods have been built
	 * into this method.
	 *
	 * @param textBlocks The list of {@link TextBlocksObject}s to populate.
	 * @param i          The item to retrieve.
	 */
	private void getTextBlocks(final ArrayList<TextBlocksObject> textBlocks, final int i) {
		final String textBlockTitle = _resourceBundle.getString("TextBlocksReportPluginProvider.contents." + i + ".title");
		final String textBlockContent = _resourceBundle.getString("TextBlocksReportPluginProvider.contents." + i + ".text");
		if (_patternSearchTerm == null || (!textBlockTitle.isEmpty() && textBlockTitle.toLowerCase().contains(_patternSearchTerm.toLowerCase())) || (!_patternSearchTitlesOnly && !textBlockContent.isEmpty() && textBlockContent.toLowerCase().contains(_patternSearchTerm.toLowerCase()))) {
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				_context.logWarning("Could not put thread to sleep in order to simulate remote data request times. Don't worry, functionality of this class is not impaired.");
			}
			textBlocks.add(new TextBlocksObject(textBlockTitle, textBlockContent, "webclient_examples/media/report/textblocks/thumbnail_dummies/tn_" + i + ".png"));
		}
	}
}
