package de.espirit.firstspirit.opt.example.webedit.report;

import de.espirit.firstspirit.access.BaseContext;
import de.espirit.firstspirit.webedit.WebeditUiAgent;
import de.espirit.firstspirit.webedit.plugin.WebeditReportPlugin;
import de.espirit.firstspirit.webedit.plugin.report.Parameter;
import de.espirit.firstspirit.webedit.plugin.report.ParameterBoolean;
import de.espirit.firstspirit.webedit.plugin.report.ParameterSelect;
import de.espirit.firstspirit.webedit.plugin.report.ParameterText;
import de.espirit.firstspirit.webedit.plugin.report.ReportPluginProvider;
import de.espirit.firstspirit.webedit.plugin.report.ReportPluginRenderer;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;


/**
 * Defines a WebClient report plug-in that uses {@link TextBlocksObject}s in order to display a list of example texts.
 */
public class TextBlocksWebeditReportPlugin implements WebeditReportPlugin<TextBlocksObject> {

	private static final String MESSAGE_BUNDLE = "de.espirit.firstspirit.opt.example.webedit.report.ReportMessages";

	private BaseContext _context;
	private ResourceBundle _resourceBundle;

	static ParameterText PATTERN_TEXT;
	static ParameterBoolean PATTERN_BOOLEAN;
	static ParameterSelect PATTERN_SELECT;


	/**
	 * Provides the report's name that will be used in several locations in the WebClient user interface as well as in
	 * Server and Project Configuration.
	 *
	 * @return A short, human-readable name to identify this report.
	 */
	@Override
	public String getTitle() {
		return _resourceBundle.getString("TextBlocksWebeditReportPlugin.name");
	}


	/**
	 * Provides a path to an icon representing this report. The parameter {@code selected} is observed in order to provide
	 * distinct URLs for "inactive" and "active" icons, relative to the project-specific WebClient web application's
	 * directory.
	 *
	 * @param selected The selected-state of the icon.
	 * @return The relative URL to the report's icon, based on the WebClient web app's directory.
	 */
	@Override
	public String getIcon(final boolean selected) {
		if (selected) {
			return "webclient_examples/media/report/textblocks/reportbutton_selected.png";
		} else {
			return "webclient_examples/media/report/textblocks/reportbutton_normal.png";
		}
	}


	/**
	 * Compile a set of parameters that will be rendered in the report's filter area so the user may limit the results this
	 * report displays.
	 *
	 * @return A {@link List} containing the text, boolean and select pattern constants configured in {@code setUp()}.
	 */
	@Override
	public List<Parameter<?>> getParameter() {
		return Arrays.<Parameter<?>>asList(PATTERN_TEXT, PATTERN_BOOLEAN, PATTERN_SELECT);
	}


	/**
	 * Instantiate a data provider ({@link TextBlocksReportPluginProvider}) and return this object.
	 *
	 * @return An instance of {@link TextBlocksReportPluginProvider}.
	 */
	@Override
	public ReportPluginProvider<TextBlocksObject> createProvider() {
		return new TextBlocksReportPluginProvider(_context);
	}


	/**
	 * Instantiate a renderer ({@link TextBlocksReportPluginRenderer}) and return this object.
	 *
	 * @return An instance of {@link TextBlocksReportPluginRenderer}.
	 */
	@Override
	public ReportPluginRenderer<TextBlocksObject> createRenderer() {
		return new TextBlocksReportPluginRenderer(_context);
	}


	/**
	 * Setup tasks: configure a resource bundle with the WebClient user interface's current locale and specify several
	 * filter parameters with localization.
	 *
	 * @param context The context the plug-in operates in.
	 */
	@Override
	public void setUp(@NotNull final BaseContext context) {
		_context = context;
		_resourceBundle = ResourceBundle.getBundle(MESSAGE_BUNDLE, context.requireSpecialist(WebeditUiAgent.TYPE).getDisplayLanguage().getLocale());

		// Configure the constant text parameter that will be used to provide a search term text field in the report's filter area.
		PATTERN_TEXT = new ParameterText("pattern_text", _resourceBundle.getString("TextBlocksWebeditReportPlugin.searchhint"), null);

		// Configure the constant boolean parameter that will be used to generate a "Search in titles only" checkbox in the report's filter area.
		PATTERN_BOOLEAN = new ParameterBoolean("pattern_boolean", _resourceBundle.getString("TextBlocksWebeditReportPlugin.searchonlytitles"), false);

		// Create several selection options in order to configure a constant select parameter that will be used to generate a dropdown list in the report's filter area, allowing the user to influence sorting behavior.
		final ArrayList<ParameterSelect.SelectItem> selectPatternItems = new ArrayList<ParameterSelect.SelectItem>();
		selectPatternItems.add(new ParameterSelect.SelectItem(_resourceBundle.getString("TextBlocksWebeditReportPlugin.sortorder.ascending"), "ascending"));
		selectPatternItems.add(new ParameterSelect.SelectItem(_resourceBundle.getString("TextBlocksWebeditReportPlugin.sortorder.descending"), "descending"));
		PATTERN_SELECT = new ParameterSelect("pattern_select", selectPatternItems, "ascending");
	}


	/**
	 * Clean-up method, used in case an object instance of this class is no longer needed.
	 */
	@Override
	public void tearDown() {
	}
}
